# -*- coding: utf-8 -*-

"""Console script for awssh."""

import click
import awssh
import subprocess
from termcolor import colored

REGION_HELP = """AWS Region, Overrides the ENV and shared config.
                 Execute 'awssh region_aliases' to see the aliases
                available VS typing full region name"""


@click.group(
        help="Helper to SSH into instances and list ips for mussh")
def main():
    """ Entry Point """
    pass


@main.command(help='List IPS for all instances and mark EIPs')
@click.option("--region", "-r",
              help="AWS Region, overrides ENV and shared config")
@click.option("--private", "-p", is_flag=True)
def ls(region=None, private=False):

    defs = {}

    if private:
        defs.update({"privateip": True})

    awsh = awssh.Awssh(region=region)
    servers = awsh.return_server_list(**defs)

    if len(servers) <= 0:
        click.echo("No ec2 instances found")
        return

    click.echo('-------------------------')
    click.echo('\t{0} = EIP'.format(colored('✓', 'green')))
    click.echo('-------------------------')

    for k, v in enumerate(servers):
        click.echo(
            '{0} [{1}]: {2}'.format(
                colored(
                    v['Eip'],
                    'green'),
                v['Ip'],
                v['Name']))


@main.command(
    help="List ip's for use with mussh. Fuzzy search enabled by default")
@click.option("--region", "-r", help=REGION_HELP)
@click.option("--exact", "-e", is_flag=True, default=False,
              help='Disable ffuzzy search and return exact Tag[Name] matches')
@click.option("--private", "-p", is_flag=True)
@click.argument('name', default='')
def ips(name, region=None, exact=False, private=False):
    defs = {}

    if private:
        defs.update({"privateip": True})

    defs.update({"exact": exact})

    awsh = awssh.Awssh(region=region)
    ips = awsh.list_ips(name, **defs)
    if len(ips) <= 0:
        return

    for ip in ips:
        click.echo(ip)


@main.command(help='SSH Into instances')
@click.option("--user", "-u", default='ec2-user', help='Forward SSH-Agent')
@click.option("--agent", "-A", is_flag=True, default=False, help='SSH User')
@click.option(
    "--tty",
    "-t",
    default=False,
    is_flag=True,
    help="TTY SSH Option")
@click.option("--region", "-r", help=REGION_HELP)
@click.option("--private", "-p", is_flag=True)
def ssh(user, tty, region=None, agent=False, private=False):
    ''' SSH Helper '''
    awsh = awssh.Awssh(region=region)

    defs = {
        'privateip': private
    }

    servers = awsh.return_server_list(**defs)

    if len(servers) <= 0:
        click.echo("No servers available")
        return

    click.echo('-------------------------')
    click.echo('\t{0} = EIP'.format(colored('✓', 'green')))
    click.echo('-------------------------')

    for k, v in enumerate(servers):
        idx = k + 1
        if idx < 10:
            idx = ' {0}'.format(idx)

        click.echo(
            '{0}) {1} [{2}]: {3}'.format(
                idx,
                colored(
                    v['Eip'],
                    'green'),
                v['Ip'],
                v['Name']))

    prompt = 'Select a server to ssh into [1-{0}]'.format(len(servers))

    ans = click.prompt(prompt, type=int)

    if ans <= 0 or ans > len(servers):
        click.echo("Invalid selection")
        return

    if ans > 0 and ans <= len(servers):
        server = servers[(ans - 1)]

        if tty:
            tty = "-t"
        else:
            tty = ""

        # agent forwarding
        if agent:
            agent = "-A"
        else:
            agent = ''

        click.echo("---------------------")
        click.echo(
            "Server: {0}({1})".format(
                server['Name'],
                server['Ip'].strip()))
        click.echo("User: {0}".format(user))
        click.echo("---------------------")
        subprocess.call(
            'ssh {2} {3} {0}@{1}'.format(
                user,
                server['Ip'].strip(),
                tty,
                agent),
            shell=True)


@main.command(help='SSH Connect via Bastion host')
@click.option("--user", "-u", default='ec2-user', help='The ssh user')
@click.option("--region", "-r", help=REGION_HELP)
def bssh(user, region):

    awsh = awssh.Awssh(region=region)
    servers = awsh.return_server_list()

    if len(servers) <= 0:
        click.echo("No servers available")
        return

    click.echo("-----------------------")
    click.echo('\t{0} = EIP'.format(colored('✓', 'green')))
    click.echo("-----------------------")

    for k, v in enumerate(servers):
        idx = k + 1
        if idx < 10:
            idx = ' {0}'.format(idx)

        click.echo(
            '{0}) {1} [{2}]: {3}'.format(
                idx,
                colored(
                    v['Eip'],
                    'green'),
                v['Ip'],
                v['Name']))

    prompt = 'Select the bastion host to proxy thru [1-{0}]'.format(
        len(servers))

    ans = click.prompt(prompt, type=int)

    if ans <= 0 or ans > len(servers):
        click.echo("Invalid selection")
        return

    bastion = servers[(ans - 1)]

    servers = awsh.return_server_list(**{'privateip': True})

    click.echo("-----------------------")
    click.echo('Private IPs')
    click.echo("-----------------------")

    for k, v in enumerate(servers):
        idx = k + 1
        if idx < 10:
            idx = ' {0}'.format(idx)

        click.echo(
            '{0}) {1} [{2}]: {3}'.format(
                idx,
                colored(
                    v['Eip'],
                    'green'),
                v['Ip'],
                v['Name']))

    prompt = 'Select the host to SSH into [1-{0}]'.format(len(servers))

    ans = click.prompt(prompt, type=int)

    if ans <= 0 or ans > len(servers):
        click.echo("Invalid selection")
        return

    server = servers[(ans - 1)]

    click.echo("---------------------")
    click.echo(
        "Bastion: {0}({1})".format(
            bastion['Name'],
            bastion['Ip'].strip()))
    click.echo(
        "Server: {0}({1})".format(
            server['Name'],
            server['Ip'].strip()))
    click.echo("User: {0}".format(user))
    click.echo("---------------------")

    cmd = "ssh -o ProxyCommand='ssh -W %h:%p {0}@{1}' {0}@{2}".format(
        user, bastion['Ip'].strip(), server['Ip'].strip())

    subprocess.call(
        cmd,
        shell=True)


@main.command(help="scp transfer files")
@click.argument("local_path", type=click.Path(exists=True))  # noqa
@click.argument("remote_path", type=click.Path(exists=False))  # noqa
@click.option("--user", "-u", default='ec2-user', help='The ssh user')
@click.option("--region", "-r", help=REGION_HELP)
def scp(local_path, remote_path, user='ec2-user', region=None):
    awsh = awssh.Awssh(region=region)

    servers = awsh.return_server_list()

    if len(servers) <= 0:
        click.echo("No servers available")
        return

    click.echo('-------------------------')
    click.echo('\t{0} = EIP'.format(colored('✓', 'green')))
    click.echo('-------------------------')

    for k, v in enumerate(servers):
        idx = k + 1
        if idx < 10:
            idx = ' {0}'.format(idx)

        click.echo(
            '{0}) {1} [{2}]: {3}'.format(
                idx,
                colored(
                    v['Eip'],
                    'green'),
                v['Ip'],
                v['Name']))

    prompt = 'Select a server to ssh into [1-{0}]'.format(len(servers))

    ans = click.prompt(prompt, type=int)

    if ans <= 0 or ans > len(servers):
        click.echo("Invalid selection")
        return

    if ans > 0 and ans <= len(servers):
        server = servers[(ans - 1)]

        click.echo("---------------------")
        click.echo(
            "Server: {0}({1})".format(
                server['Name'],
                server['Ip'].strip()))
        click.echo("User: {0}".format(user))
        click.echo('Local Path: %s' % click.format_filename(local_path))  # noqa
        click.echo('Remote Path:{0}'.format(remote_path))
        click.echo("---------------------")
        subprocess.call('scp -r {0} {2}@{3}:{1}'.format(local_path,
                                                        remote_path, user, server['Ip'].strip()), shell=True)  # noqa


@main.command(help="View region aliases")
def region_aliases():

    awsh = awssh.Awssh()
    aliases = awsh.alias_regions()

    for k, v in aliases[0].items():
        click.echo("{0} = {1}".format(k, v))


# @main.command()
# def tester():

    # assh = awssh.Awssh(region='uw2')
    # print(awssh.Awssh.get_region())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import boto3
import os
import jmespath

""" Exceptions """

""" Boto client error """


class AwsClientErr(Exception):
    pass


""" Ec2 service error """


class Ec2Err(Exception):
    pass


class Ec2DescribeInstancesErr(Exception):
    pass


"""Main module."""


class Awssh(object):

    _clients = {}
    _default_region = 'us-west-2'
    _region = None

    def __init__(self, region=None):

        if region:
            self.set_region(region)
            # Awssh._region = region
        else:
            Awssh._region = Awssh.default_region()

    def client(self, service):

        client_key = '{0}-{1}'.format(service, Awssh.get_region())
        if not Awssh._clients.get(client_key):

            try:
                client = boto3.client(service, region_name=Awssh.get_region()) or False  # noqa
            except:
                raise AwsClientErr("Unable to connect to AWS API: {0}".format(service))  # noqa

            if not client:
                raise AwsClientErr(
                    'Could not create service: {0}'.format(service))

            Awssh._clients.update({client_key: client})

        return Awssh._clients.get(client_key)

    @staticmethod
    def default_region():

        if os.environ.get("AWS_DEFAULT_REGION") is not None:
            return os.environ.get("AWS_DEFAULT_REGION")

        return Awssh._default_region

    @staticmethod
    def get_region():
        return Awssh._region

    def set_region(self, region):
        try:
            regions = self.alias_regions()
        except:
            Awssh._region = region
            return

        if regions[0].get(region):
            region = regions[0].get(region)

        if region not in regions[1]:
            new_region = Awssh._default_region
        else:
            new_region = region

        Awssh._region = new_region

    def alias_regions(self):
        ec2 = self.client("ec2")
        r = ec2.describe_regions()
        region_names = jmespath.search("Regions[].RegionName", r)
        regions = []
        aliases = {}

        for r in region_names:
            spl = r.split('-')
            key = '{0}{1}{2}'.format(spl[0][:1], spl[1][:1], spl[2][:1])
            aliases.update({key: r})
            regions.append(r)

        return aliases, regions

    def return_ec2_servers(self, **kwargs):

        ec2Client = self.client('ec2')
        ipPath = 'PublicIpAddress'

        if kwargs.get('privateip'):
            ipPath = 'PrivateIpAddress'

        try:
            inst = ec2Client.describe_instances()
        except:
            raise Ec2DescribeInstancesErr("Unable to connect to api")

        ips = []

        # compile JMESpath queries
        jmespath.compile("Tags[?Key=='Name'].Value")
        jmespath.compile("PublicIpAddress")
        jmespath.compile("PrivateIpAddress")

        for i in inst['Reservations']:
            for ii in i['Instances']:
                if ii['State']['Code'] not in [16]:
                    continue

                try:
                    name = jmespath.search("Tags[?Key=='Name'].Value", ii)[0]
                except:
                    name = 'N/A'

                try:
                    ip = jmespath.search(ipPath, ii)
                except:
                    ip = False

                if ip:
                    ips.append({'Name': name, 'Ip': ip})

        return ips

    def return_elastic_ips(self, **kwargs):

        ec2Client = self.client('ec2')

        ips = []
        try:
            eips = ec2Client.describe_addresses()
        except:
            exit("Unable to connect to AWS API")

        for ip in eips['Addresses']:
            if 'PublicIp' in ip:
                ips.append(ip['PublicIp'])

        return ips

    def return_server_list(self, **kwargs):

        servers = self.return_ec2_servers(**kwargs)
        eips = self.return_elastic_ips()

        for k, v in enumerate(servers):
            tag = ' '
            if v['Ip'] in eips:
                tag = '✓'
            # pad the ips for uniformity
            while len(servers[k]['Ip']) < 15:
                servers[k]['Ip'] = ' {0}'.format(v['Ip'])
            servers[k].update({'Eip': tag})

        def cmp(x, y):
            if x['Name'] < y['Name']:
                return -1
            return 0

        servers = sorted(servers, cmp=cmp)

        return servers

    def list_ips(self, name, **kwargs):

        exact = False

        if kwargs.get('exact'):
            exact = kwargs['exact']

        servers = self.return_ec2_servers(**kwargs)

        ips = []

        if len(servers) <= 0:
            return ips

        name = name.lower()

        for k, v in enumerate(servers):
            if len(name) > 0:
                if exact:
                    if name == v['Name'].lower():
                        ips.append(v['Ip'])
                else:
                    if name in v['Name'].lower():
                        ips.append(v['Ip'])
            else:
                ips.append(v['Ip'])

        return ips



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `awssh` package."""

import pytest
import mock
import os
import boto3
import json
import sys
import botocore

from click.testing import CliRunner

from awssh import awssh


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


@pytest.fixture
def ec2_api_mock():
    def mock_api(self, serv, arg, **kwargs):
        if serv == 'DescribeInstances':
            return {'Reservations': [
                {'Instances': [
                    {
                        'State': {'Code': 16},
                        'Tags': [
                            {'Key': 'Name', 'Value': 'Name1'},
                            {'Key': 'Bunk', 'Value': 'Ugh'}
                        ],
                        'PublicIpAddress': '1.1.1.1',
                        'PrivateIpAddress': '2.2.2.2'
                    },
                    {
                        'State': {'Code': 16},
                        'Tags': [
                            {'Key': 'Name', 'Value': 'Name2'},
                            {'Key': 'Bunk', 'Value': 'Ugh'}
                        ],
                        'PublicIpAddress': '3.3.3.3',
                        'PrivateIpAddress': '4.4.4.4'
                    },
                ]},
                {'Instances': [
                    {
                        'State': {'Code': 3},
                        'Tags': [
                            {'Key': 'Name', 'Value': 'Name3'},
                            {'Key': 'Bunk', 'Value': 'Ugh'}
                        ],
                        'PublicIpAddress': '5.5.5.5',
                        'PrivateIpAddress': '6.6.6.6'
                    },
                    {
                        'State': {'Code': 16},
                        'Tags': [
                            {'Key': 'NameNope', 'Value': 'Name4'},
                            {'Key': 'Bunk', 'Value': 'Ugh'}
                        ],
                        'PublicIpAddress': '7.7.7.7',
                        'PrivateIpAddress': '8.8.8.8'
                    },
                ]},
                {'Instances': [
                    {
                        'State': {'Code': 16},
                        'Tags': [
                            {'Key': 'Name', 'Value': 'Name5'},
                            {'Key': 'Bunk', 'Value': 'Ugh'}
                        ],
                        'PublicIpAddress': '9.9.9.9',
                        'PrivateIpAddress': '10.10.10.10'
                    },
                    {
                        'State': {'Code': 16},
                        'Tags': [
                            {'Key': 'Name', 'Value': 'NoIp'},
                            {'Key': 'Bunk', 'Value': 'Ugh'}
                        ],
                        'PublicIpAddress': '',
                        'PrivateIpAddress': ''
                    },
                ]},
            ]}
        if serv == 'DescribeRegions':
            return {'Regions': [{'Endpoint': 'ec2.ap-south-1.amazonaws.com',
                                 'RegionName': 'ap-south-1'},
                                {'Endpoint': 'ec2.eu-west-2.amazonaws.com',
                                 'RegionName': 'eu-west-2'},
                                {'Endpoint': 'ec2.eu-west-1.amazonaws.com',
                                 'RegionName': 'eu-west-1'},
                                {'Endpoint': 'ec2.ap-northeast-2.amazonaws.com', # noqa
                                 'RegionName': 'ap-northeast-2'},
                                {'Endpoint': 'ec2.ap-northeast-1.amazonaws.com', # noqa
                                 'RegionName': 'ap-northeast-1'},
                                {'Endpoint': 'ec2.sa-east-1.amazonaws.com',
                                 'RegionName': 'sa-east-1'},
                                {'Endpoint': 'ec2.ca-central-1.amazonaws.com',
                                 'RegionName': 'ca-central-1'},
                                {'Endpoint': 'ec2.ap-southeast-1.amazonaws.com',  # noqa
                                 'RegionName': 'ap-southeast-1'},
                                {'Endpoint': 'ec2.ap-southeast-2.amazonaws.com',  # noqa
                                 'RegionName': 'ap-southeast-2'},
                                {'Endpoint': 'ec2.eu-central-1.amazonaws.com',
                                 'RegionName': 'eu-central-1'},
                                {'Endpoint': 'ec2.us-east-1.amazonaws.com',
                                 'RegionName': 'us-east-1'},
                                {'Endpoint': 'ec2.us-east-2.amazonaws.com',
                                 'RegionName': 'us-east-2'},
                                {'Endpoint': 'ec2.us-west-1.amazonaws.com',
                                 'RegionName': 'us-west-1'},
                                {'Endpoint': 'ec2.us-west-1.amazonaws.com',
                                 'RegionName': 'zz-iraq-1'},
                                {'Endpoint': 'ec2.us-west-2.amazonaws.com',
                                 'RegionName': 'us-west-2'}],
                    'ResponseMetadata': {'RetryAttempts': 0,
                                         'HTTPStatusCode': 200,
                                         'RequestId': '48139461-f0cf-4fc1-9da6-c55016b31d90',  # noqa
                                         'HTTPHeaders': {'transfer-encoding': 'chunked',  # noqa
                                                         'vary': 'Accept-Encoding',  # noqa
                                                         'server': 'AmazonEC2',
                                                         'content-type': 'text/xml;charset=UTF-8',  # noqa
                                                         'date': 'Thu, 19 Oct 2017 05:39:31 GMT'}}} # noqa

    return mock_api


def test_content(response):
    """Sample pytest test function with the pytest fixture as an argument."""
    # from bs4 import BeautifulSoup
    # assert 'GitHub' in BeautifulSoup(response.content).title.string


def test_command_line_interface():
    """Test the CLI."""
    # runner = CliRunner()
    # result = runner.invoke(awssh.cli.main)
    # assert result.exit_code == 0
    # # assert 'awssh.cli.main' in result.output
    # help_result = runner.invoke(awssh.cli.main, ['--help'])
    # assert help_result.exit_code == 0
    # assert '--help' in help_result.output


@mock.patch('awssh.awssh.boto3')
def test_awssh_client(my_mock):

    awh = awssh.Awssh()

    awh.client("swf")
    my_mock.client.assert_called_with("swf", region_name='us-west-2')

    os.environ['AWS_DEFAULT_REGION'] = 'iraq-west-1'

    awh = awssh.Awssh()
    awh.client('rds')
    my_mock.client.assert_called_with("rds", region_name='iraq-west-1')

    awssh.Awssh._clients = {}
    awssh.Awssh._region = None
    del os.environ['AWS_DEFAULT_REGION']

    awssh.Awssh._clients = {}
    awssh.Awssh._region = None
    awh = awssh.Awssh(region='us-east-2')

    awh.client('sns')
    my_mock.client.assert_called_with('sns', region_name='us-east-2')

    my_mock.client.return_value = False
    with pytest.raises(Exception) as exe:
        awh.client('sqs')
    assert "Could not create" in str(exe.value)

    my_mock.client.side_effect = Exception("Test")
    with pytest.raises(Exception) as exe:
        awh.client("sts")
    assert "Unable to connect" in str(exe.value)

    awssh.Awssh._clients = {}
    awssh.Awssh._region = None


def test_return_ec2_servers(ec2_api_mock):

    with mock.patch('botocore.client.BaseClient._make_api_call', new=ec2_api_mock): # noqa
        a = awssh.Awssh()

        res = a.return_ec2_servers()

        test_str = json.dumps(res)

        assert '5.5.5.5' not in test_str
        assert 'NoIp' not in test_str
        assert 'Name5' in test_str
        assert '9.9.9.9' in test_str


def test_set_region(ec2_api_mock):

    with mock.patch('botocore.client.BaseClient._make_api_call', new=ec2_api_mock): # noqa

        a = awssh.Awssh()

        a.set_region('zi1')
        assert awssh.Awssh.get_region() == 'zz-iraq-1'

        a.set_region('test-error')
        assert awssh.Awssh.get_region() == 'us-west-2'

        a.set_region('us-east-2')
        assert awssh.Awssh.get_region() == 'us-east-2'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import os
from setuptools import setup, find_packages
from pip.req import parse_requirements

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()


pkg_reqs = [
            'Click',
            'boto3',
            'termcolor',
            'jmespath'
        ]

setup_requirements = [
    'pytest-runner',
    'Click',
    'boto3',
    'termcolor',
    'jmespath',
    # TODO(ibejohn818): put setup requirements (distutils extensions, etc.) here
]

test_requirements = [
    'pytest',
    'Click',
    'boto3',
    'termcolor',
    'jmespath',
    'mock'
    # TODO: put package test requirements here
]

setup(
    name='jh-awssh',
    version='0.22.0',
    description="SSH Connect to Ec2 instances",
    long_description=readme + '\n\n' + history,
    author="John Hardy",
    author_email='john.hardy@me.com',
    url='https://github.com/ibejohn818/awssh',
    packages=find_packages(include=['awssh']),
    entry_points={
        'console_scripts': [
            'awssh=awssh.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=pkg_reqs,
    license="MIT license",
    zip_safe=False,
    keywords='awssh',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    setup_requires=setup_requirements,
)
